package de.intevation.flys.artifacts.model.sq;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.math.MathException;

import org.apache.commons.math.stat.descriptive.moment.StandardDeviation;

import org.apache.log4j.Logger;

import de.intevation.artifacts.GlobalContext;
import de.intevation.artifacts.common.utils.Config;
import de.intevation.flys.artifacts.context.FLYSContext;
import de.intevation.flys.artifacts.math.GrubbsOutlier;
import de.intevation.flys.artifacts.math.StdDevOutlier;

public class Outlier
{
    private static Logger log = Logger.getLogger(Outlier.class);

    private static final String OUTLIER_METHOD =
        "/artifact-database/options/minfo-sq/outlier-method/@name";

    private static final String GRUBBS = "grubbs";

    private static final String STD_DEV = "std-dev";

    public interface Callback {

        void initialize(List<SQ> sqs) throws MathException;

        double eval(SQ sq);

        void iterationFinished(
            double   stdDev,
            SQ       outlier,
            List<SQ> remaining);

    } // interface Callback

    public static void detectOutliers(
        Callback callback,
        List<SQ> sqs,
        double   stdDevFactor
    )
    throws MathException
    {
        boolean debug = log.isDebugEnabled();

        if (debug) {
            log.debug("stdDevFactor: " + stdDevFactor);
        }
        String method = Config.getStringXPath(OUTLIER_METHOD);
        log.debug("method: " + method);
        if (method == null) {
            method = "std-dev";
        }
        List<SQ> data = new ArrayList<SQ>(sqs);

        while (data.size() > 2) {

            callback.initialize(data);

            List<Double> values = new ArrayList<Double>();
            for (SQ sq: data) {
                values.add(callback.eval(sq));
            }

            Integer ndx = null;
            double[] stdDev = new double[1];
            if (method.equals(GRUBBS)) {
                ndx = GrubbsOutlier.findOutlier(values, stdDevFactor/100, stdDev);
            }
            else {
                ndx = StdDevOutlier.findOutlier(values, stdDevFactor, stdDev);
            }
            if (ndx == null) {
                callback.iterationFinished(stdDev[0], null, data);
                break;
            }

            SQ outlier = data.remove((int)ndx);
            log.debug("stdDev: " + stdDev[0]);
            log.debug("removed " + ndx + "; S: " + outlier.getS() + " Q: " + outlier.getQ());
            callback.iterationFinished(stdDev[0], outlier, data);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
